/* eslint-disable import/no-unresolved, import/extensions */
import { StyleSheet } from 'react-native';
import { arrayEach } from 'fast-loops';
import { processStyleWithPlugins, RULE_TYPE, CLEAR_TYPE } from 'fela-utils';
export function createRenderer() {
  var config = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var renderer = {
    listeners: [],
    cache: {},
    plugins: config.plugins || [],
    isNativeRenderer: true,
    clear: function clear() {
      renderer.cache = {};

      renderer._emitChange({
        type: CLEAR_TYPE
      });
    },
    subscribe: function subscribe(callback) {
      renderer.listeners.push(callback);
      return {
        unsubscribe: function unsubscribe() {
          return renderer.listeners.splice(renderer.listeners.indexOf(callback), 1);
        }
      };
    },
    renderRule: function renderRule(rule) {
      var props = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
      var processedStyle = processStyleWithPlugins(renderer, rule(props, renderer), RULE_TYPE, props);
      var reference = JSON.stringify(processedStyle);

      if (!renderer.cache.hasOwnProperty(reference)) {
        renderer.cache[reference] = StyleSheet.create({
          style: processedStyle
        });

        renderer._emitChange({
          type: RULE_TYPE,
          style: processedStyle
        });
      }

      return renderer.cache[reference].style;
    },
    _emitChange: function _emitChange(change) {
      arrayEach(renderer.listeners, function (listener) {
        return listener(change);
      });
    }
  };

  if (config.enhancers) {
    arrayEach(config.enhancers, function (enhancer) {
      renderer = enhancer(renderer);
    });
  }

  return renderer;
}