"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createRenderer = createRenderer;

var _reactNative = require("react-native");

var _fastLoops = require("fast-loops");

var _felaUtils = require("fela-utils");

/* eslint-disable import/no-unresolved, import/extensions */
function createRenderer() {
  var config = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var renderer = {
    listeners: [],
    cache: {},
    plugins: config.plugins || [],
    isNativeRenderer: true,
    clear: function clear() {
      renderer.cache = {};

      renderer._emitChange({
        type: _felaUtils.CLEAR_TYPE
      });
    },
    subscribe: function subscribe(callback) {
      renderer.listeners.push(callback);
      return {
        unsubscribe: function unsubscribe() {
          return renderer.listeners.splice(renderer.listeners.indexOf(callback), 1);
        }
      };
    },
    renderRule: function renderRule(rule) {
      var props = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
      var processedStyle = (0, _felaUtils.processStyleWithPlugins)(renderer, rule(props, renderer), _felaUtils.RULE_TYPE, props);
      var reference = JSON.stringify(processedStyle);

      if (!renderer.cache.hasOwnProperty(reference)) {
        renderer.cache[reference] = _reactNative.StyleSheet.create({
          style: processedStyle
        });

        renderer._emitChange({
          type: _felaUtils.RULE_TYPE,
          style: processedStyle
        });
      }

      return renderer.cache[reference].style;
    },
    _emitChange: function _emitChange(change) {
      (0, _fastLoops.arrayEach)(renderer.listeners, function (listener) {
        return listener(change);
      });
    }
  };

  if (config.enhancers) {
    (0, _fastLoops.arrayEach)(config.enhancers, function (enhancer) {
      renderer = enhancer(renderer);
    });
  }

  return renderer;
}